#!/usr/bin/env python
import os
import subprocess
from string import Template
from .command import Command


def _connection_string(dbms, connection, user, password):
    if dbms.lower() == 'oracle':
        return user + '/' + password + '@' + connection


def _sql_script(file_path):
    file = open(file_path, 'r')
    script = file.read()
    script += '\nshow error'
    return script


class RunSqlScript(Command):

    def _add_arguments(self, parser):
        parser.add_argument(
            '-s', '--dbms',
            type=str,
            required=True,
            help='oracle or mssql')

        parser.add_argument(
            '-d', '--directory',
            type=str,
            required=True,
            help='Directory containing script')

        parser.add_argument(
            '-f', '--file',
            type=str,
            required=True,
            help='Script file name')

        parser.add_argument(
            '-c', '--connection',
            type=str,
            required=True,
            help='db connection string')

        parser.add_argument(
            '-u', '--user',
            type=str,
            required=True,
            help='Database user')

        parser.add_argument(
            '-p', '--password',
            type=str,
            required=True,
            help='Password')

    def _runScript(self, file_path, dbms, connection):
        script = _sql_script(file_path)

        os.chdir(os.path.dirname(file_path))

        if dbms.lower() == 'oracle':
            process = subprocess.Popen(
                ['sqlplus', '-S', '-L', connection],
                stdin=subprocess.PIPE,
                stderr=subprocess.PIPE)
            process.stdin.write(script.encode('utf-8'))
            process.stdin.close()
            process.wait()

    def _execute(self):
        file_path = os.path.join(self.args.directory, self.args.file)

        message = Template(
            'Matador: Executing ${file} against ${connection} \n')
        substitutions = {
            'file': os.path.basename(file_path),
            'connection': self.args.connection
        }
        self._logger.info(message.substitute(substitutions))

        connection_string = _connection_string(
            self.args.dbms,
            self.args.connection, self.args.user, self.args.password)
        self._runScript(file_path, self.args.dbms, connection_string)



#!/usr/bin/env python
from .command import Command
from matador import utils
import subprocess
import glob
import os
import shutil


class DeployTicket(Command):

    def _add_arguments(self, parser):
        parser.prog = 'matador deploy-ticket'
        parser.add_argument(
            '-e', '--environment',
            type=str,
            required=True,
            help='Agresso environment name')

        parser.add_argument(
            '-t', '--ticket',
            type=str,
            required=True,
            help='Ticket name')

        parser.add_argument(
            '-b', '--branch',
            type=str,
            default='master',
            help='Branch name')

        parser.add_argument(
            '-p', '--package',
            type=bool,
            default=False,
            help='Whether this deployment is part of a package')

    def _checkout_ticket(self, repo_folder, ticket_folder, branch):
        subprocess.run([
            'git', '-C', repo_folder, 'checkout', branch],
            stderr=subprocess.STDOUT,
            stdout=open(os.devnull, 'w'))
        src = os.path.join(repo_folder, 'deploy', 'tickets', self.args.ticket)
        shutil.copytree(src, ticket_folder)

    def _cleanup(self, ticket_folder):
        shutil.rmtree(ticket_folder)

    def _execute(self):
        project = utils.project()
        repo_folder = utils.matador_repository_folder(project)
        ticket_folder = os.path.join(
            utils.matador_ticket_folder(project, self.args.environment),
            self.args.ticket)

        if not self.args.package:
            utils.update_repository(project, self.args.branch)
        self._checkout_ticket(repo_folder, ticket_folder, self.args.branch)

        os.chdir(ticket_folder)
        import deploy

        self._cleanup(ticket_folder)



#!/usr/bin/env python3

""" Keyword Substitution Script for Git
Based on the work by Peter Krusche at https://gist.github.com/pkrusche/7369262

Transforms stdin to expand some keywords with git version/date information.

Requires Python 3.

Specify --clean to remove this information before commit.

Setup:

1. Copy substitution.py into your git repository

2. Run:

 git config filter.substitution.smudge 'python3 substitution.py'
 git config filter.substitution.clean  'python3 substitution.py --clean'
 git add substitution.py

3. Add filters to .gitattributes:
e.g. to use substitution for all sql files:

 echo '*.sql filter=substitution' >> .gitattributes

3. Add substitution keywords to files as required:

 version:
 date:

4. Update the substitutions by deleting and then checking out the file.
"""
import sys
import subprocess
import re


def main():
    clean = False

    if len(sys.argv) > 1:
        if sys.argv[1] == '--clean':
            clean = True

    substitutions = {
        'version': None,
        'date': None,
    }

    if not clean:
        substitutions = {
            'version': subprocess.getoutput('git describe --always'),
            'date': subprocess.getoutput('git log --pretty=format:"%ad" -1'),
        }

        for key, value in substitutions.items():
            value = re.sub(r'[\n\r\t"\"]', '', value)

        for line in sys.stdin:
            for key, value in substitutions.items():
                rexp = '%s:' % key
                line = re.sub(rexp, '%s: %s' % (key, value), line)
            sys.stdout.write(line)

    else:
        for line in sys.stdin:
            for key in substitutions:
                rexp = '%s:.*' % key
                line = re.sub(rexp, '%s:' % key, line)
            sys.stdout.write(line)

if __name__ == '__main__':
    main()



#!/usr/bin/env python
import os
import subprocess
import yaml


def matador_project_folder(project):
    project_path = os.path.expanduser(
        '~/.matador/%s' % project)
    os.makedirs(project_path, exist_ok=True)
    return project_path


def matador_repository_folder(project):
    repository_path = os.path.join(
        matador_project_folder(project), 'repository')
    os.makedirs(repository_path, exist_ok=True)
    return repository_path


def matador_environment_folder(project, environment):
    working_path = os.path.join(
        matador_project_folder(project), environment)
    os.makedirs(working_path, exist_ok=True)
    return working_path


def matador_ticket_folder(project, environment):
    ticket_path = os.path.join(
        matador_environment_folder(project, environment), 'tickets')
    os.makedirs(ticket_path, exist_ok=True)
    return ticket_path


def is_git_repository(path='.'):
    return subprocess.run(
        ['git', '-C', path, 'status'],
        stderr=subprocess.STDOUT,
        stdout=open(os.devnull, 'w')) == 0


def project_folder(path='.'):
    git_output = subprocess.check_output(
        ['git', '-C', path, 'rev-parse', '--show-toplevel'],
        stderr=subprocess.STDOUT)
    return git_output.decode('utf-8').strip('\n')


def project():
    return os.path.basename(project_folder())


def environments():
    file_path = os.path.join(
        project_folder(), 'config', 'environments.yml')
    file = open(file_path, 'r')
    return yaml.load(file)


def update_repository(project, branch='master'):
    proj_folder = project_folder()
    repo_folder = matador_repository_folder(project)

    if not is_git_repository(repo_folder):
        subprocess.run([
            'git', '-C', repo_folder, 'init'],
            stderr=subprocess.STDOUT,
            stdout=open(os.devnull, 'w'))

        subprocess.run([
            'git', '-C', repo_folder, 'config', 'core.sparsecheckout', 'true'],
            stderr=subprocess.STDOUT,
            stdout=open(os.devnull, 'w'))

        subprocess.run([
            'git', '-C', repo_folder, 'remote', 'add', 'origin', proj_folder],
            stderr=subprocess.STDOUT,
            stdout=open(os.devnull, 'w'))

        git_path = (os.path.join(repo_folder, '.git'))
        config_file = os.path.join(git_path, 'config')
        with open(config_file, 'a') as f:
            f.write('[filter "substitution"]\n')
            f.write('        smudge = matador substitute-keywords\n')
            f.write('        clean = matador clean-keywords\n')
            f.close()

        attributes_file = os.path.join(git_path, 'info', 'attributes')
        with open(attributes_file, 'a') as f:
            f.write('src/ filter=substitution\n')
            f.close()

        sparse_checkout_file = os.path.join(
            git_path, 'info', 'sparse-checkout')
        with open(sparse_checkout_file, 'a') as f:
            f.write('/src\n')
            f.write('/deploy\n')
            f.close()

    subprocess.run(
        ['git', '-C', repo_folder, 'fetch', '-a'],
        stderr=subprocess.STDOUT,
        stdout=open(os.devnull, 'w'))



from .hello import Hello
from .run_sql_script import RunSqlScript
from .deploy_ticket import DeployTicket
from .substitution import SubstituteKeywords, CleanKeywords

commands = {
    'hello': Hello,
    'run-sql-script': RunSqlScript,
    'deploy-ticket': DeployTicket,
    'substitute-keywords': SubstituteKeywords,
    'clean-keywords': CleanKeywords
}



from .command import Command
import sys
import re
import subprocess


class SubstituteKeywords(Command):

    def _execute(self):
        substitutions = {
            'version': subprocess.check_output(
                ['git', 'describe', '--always'],
                stderr=subprocess.STDOUT),
            'date': subprocess.check_output(
                ['git', 'log', '--pretty=format:"%ad"', '-1']
                stderr=subprocess.STDOUT),
        }
        for key, value in substitutions.items():
            value = re.sub(r'[\n\r\t"\"]', '', value)

        for line in sys.stdin:
            for key, value in substitutions.items():
                rexp = '%s:' % key
                line = re.sub(rexp, '%s: %s' % (key, value), line)
            sys.stdout.write(line)


class CleanKeywords(Command):

    def _execute(self):
        substitutions = {
            'version': None,
            'date': None,
        }

        for line in sys.stdin:
            for key in substitutions:
                rexp = '%s:.*' % key
                line = re.sub(rexp, '%s:' % key, line)
            sys.stdout.write(line)
